@@ -19,6 +19,9 @@ namespace cpp11 {
 
 template <>
 inline SEXP r_vector<int>::valid_type(SEXP data) {
+  if (data == nullptr) {
+    throw type_error(INTSXP, NILSXP);
+  }
   if (TYPEOF(data) != INTSXP) {
     throw type_error(INTSXP, TYPEOF(data));
   }
@@ -32,7 +35,8 @@ inline int r_vector<int>::operator[](const R_xlen_t pos) const {
 }
 
 template <>
-inline int* r_vector<int>::get_p(bool is_altrep, SEXP data) {
+inline typename r_vector<int>::underlying_type* r_vector<int>::get_p(bool is_altrep,
+                                                                     SEXP data) {
   if (is_altrep) {
     return nullptr;
   } else {
@@ -141,4 +145,36 @@ inline int na() {
   return NA_INTEGER;
 }
 
+// forward declarations
+typedef r_vector<double> doubles;
+
+template <>
+bool is_na(const double& x);
+
+template <>
+double r_vector<double>::operator[](const R_xlen_t pos) const;
+
+inline integers as_integers(sexp x) {
+  if (TYPEOF(x) == INTSXP) {
+    return as_cpp<integers>(x);
+  } else if (TYPEOF(x) == REALSXP) {
+    doubles xn = as_cpp<doubles>(x);
+    R_xlen_t len = xn.size();
+    writable::integers ret(len);
+    for (R_xlen_t i = 0; i < len; ++i) {
+      double el = xn[i];
+      if (is_na(el)) {
+        ret[i] = na<int>();
+      } else if (is_convertible_without_loss_to_integer(el)) {
+        ret[i] = static_cast<int>(el);
+      } else {
+        throw std::runtime_error("All elements must be integer-like");
+      }
+    }
+    return ret;
+  }
+
+  throw type_error(INTSXP, TYPEOF(x));
+}
+
 }  // namespace cpp11
